@@ -198,7 +198,7 @@ EXPORTED void mboxevent_setnamespace(struct namespace *n)
 {
     namespace = *n;
     /* standardize IMAP URL format */
-    namespace.isadmin = 0;
+    namespace.isadmin = 1;
 }
 
 static int mboxevent_enabled_for_mailbox(struct mailbox *mailbox)
@@ -619,12 +619,12 @@ void mboxevent_add_flag(struct mboxevent *event, const char *flag)
 EXPORTED void mboxevent_set_access(struct mboxevent *event,
 				   const char *serveraddr, const char *clientaddr,
 				   const char *userid, const char *mailboxname,
-				   const int ext_name)
+				   const int ext_name __attribute__((unused)))
 {
     char url[MAX_MAILBOX_PATH+1];
     struct imapurl imapurl;
-    char extname[MAX_MAILBOX_NAME];
-    char *userbuf;
+    mbentry_t *mbentry;
+    int r;
 
     if (!event)
 	return;
@@ -636,33 +636,26 @@ EXPORTED void mboxevent_set_access(struct mboxevent *event,
     }
 
     /* all events needs uri parameter */
-    if (!event->params[EVENT_URI].filled) {
-	memset(&imapurl, 0, sizeof(struct imapurl));
-	imapurl.server = config_servername;
+    memset(&imapurl, 0, sizeof(struct imapurl));
+    imapurl.server = config_servername;
 
-	if (mailboxname != NULL) {
-	    /* translate internal mailbox name to external */
-	    assert(namespace.mboxname_toexternal != NULL);
-	    (*namespace.mboxname_toexternal)(&namespace, mailboxname,
-					     mboxname_to_userid(mailboxname),
-					     extname);
+    if (mailboxname != NULL) {
+	struct mboxname_parts parts;
+	mboxname_to_parts(mailboxname, &parts);
 
-	    /* translate any separators in user */
-	    userbuf = (char *)mboxname_to_userid(mailboxname);
-	    if (userbuf != NULL && ext_name)
-		mboxname_hiersep_toexternal(&namespace, userbuf,
-					    config_virtdomains ? strcspn(userbuf, "@") : 0);
+	imapurl.mailbox = parts.box;
+    }
 
-	    imapurl.mailbox = extname;
-	    imapurl.user = userbuf;
-	}
+    imapurl_toURL(url, &imapurl);
 
-	imapurl_toURL(url, &imapurl);
-	FILL_STRING_PARAM(event, EVENT_URI, xstrdup(url));
-
-	if (event->type & MAILBOX_EVENTS) {
-	    FILL_STRING_PARAM(event, EVENT_MAILBOX_ID, xstrdup(url));
-	}
+    r = mboxlist_lookup(mailboxname, &mbentry, NULL);
+    if (r) {
+	/* couldn't look up mailboxname -- fall back to old url behaviour */
+	FILL_STRING_PARAM(event, EVENT_MAILBOX_ID, xstrdup(url));
+    }
+    else {
+	FILL_STRING_PARAM(event, EVENT_MAILBOX_ID, xstrdup(mbentry->uniqueid));
+	mboxlist_entry_free(&mbentry);
     }
 
     if (serveraddr && mboxevent_expected_param(event->type, EVENT_SERVER_ADDRESS)) {
@@ -673,12 +666,10 @@ EXPORTED void mboxevent_set_access(struct mboxevent *event,
     }
     if (userid && mboxevent_expected_param(event->type, EVENT_USER)) {
 	/* translate any separators in user */
-	userbuf = xstrdup(userid);
-	if (userbuf != NULL && ext_name)
-	    mboxname_hiersep_toexternal(&namespace, userbuf,
-				    config_virtdomains ? strcspn(userbuf, "@") : 0);
-
-	FILL_STRING_PARAM(event, EVENT_USER, userbuf);
+	char *user = xstrdup(userid);
+	mboxname_hiersep_toexternal(&namespace, user,
+				    config_virtdomains ? strcspn(user, "@") : 0);
+	FILL_STRING_PARAM(event, EVENT_USER, user); /* note, absorbs the user */
     }
 }
 
@@ -893,7 +884,6 @@ void mboxevent_extract_quota(struct mboxevent *event, const struct quota *quota,
     struct imapurl imapurl;
     char url[MAX_MAILBOX_PATH+1];
     char extname[MAX_MAILBOX_NAME];
-    char *userbuf;
 
     if (!event)
 	return;
@@ -925,23 +915,23 @@ void mboxevent_extract_quota(struct mboxevent *event, const struct quota *quota,
      * It seems that it does not correspond to the concept of
      * quota root specified in RFC 2087. Thus we fill uri with quota root
      */
-    if (event->type & QUOTA_EVENTS) {
-	/* translate internal mailbox name to external */
-	assert(namespace.mboxname_toexternal != NULL);
-	(*namespace.mboxname_toexternal)(&namespace, quota->root,
-					 mboxname_to_userid(quota->root),
-					 extname);
-
-	/* translate any separators in user */
-	userbuf = (char *)mboxname_to_userid(quota->root);
-	if (userbuf != NULL)
-	    mboxname_hiersep_toexternal(&namespace, userbuf,
-					config_virtdomains ? strcspn(userbuf, "@") : 0);
+    if (!event->params[EVENT_URI].filled && event->type & QUOTA_EVENTS) {
+	char *user = (char *)mboxname_to_userid(quota->root);
 
 	memset(&imapurl, 0, sizeof(struct imapurl));
 	imapurl.server = config_servername;
+
+	/* translate internal mailbox name to external */
+	assert(namespace.mboxname_toexternal != NULL);
+	(*namespace.mboxname_toexternal)(&namespace, quota->root, NULL, extname);
 	imapurl.mailbox = extname;
-	imapurl.user = userbuf;
+
+	if (user) {
+	    /* translate any separators in user */
+	    mboxname_hiersep_toexternal(&namespace, user,
+					config_virtdomains ? strcspn(user, "@") : 0);
+	}
+
 	imapurl_toURL(url, &imapurl);
 
 	if (!event->params[EVENT_URI].filled) {
@@ -955,7 +945,7 @@ void mboxevent_extract_quota(struct mboxevent *event, const struct quota *quota,
 	 */
 
 	if (!event->params[EVENT_USER].filled) {
-	    FILL_STRING_PARAM(event, EVENT_USER, (userbuf == NULL ? '\0' : xstrdup(userbuf)));
+	    FILL_STRING_PARAM(event, EVENT_USER, xstrdupsafe(user));
 	}
     }
 }
@@ -982,7 +972,6 @@ EXPORTED void mboxevent_extract_mailbox(struct mboxevent *event,
     struct imapurl imapurl;
     char url[MAX_MAILBOX_PATH+1];
     char extname[MAX_MAILBOX_NAME];
-    char *userbuf;
 
     if (!event)
 	return;
@@ -998,22 +987,14 @@ EXPORTED void mboxevent_extract_mailbox(struct mboxevent *event,
     }
 
     /* translate internal mailbox name to external */
-    assert(namespace.mboxname_toexternal != NULL);
-    (*namespace.mboxname_toexternal)(&namespace, mailbox->name,
-				     mboxname_to_userid(mailbox->name), extname);
-
-    /* translate any separators in user */
-    userbuf = (char *)mboxname_to_userid(mailbox->name);
-    if (userbuf != NULL)
-	mboxname_hiersep_toexternal(&namespace, userbuf,
-				    config_virtdomains ? strcspn(userbuf, "@") : 0);
-
-    /* all events needs uri parameter */
     memset(&imapurl, 0, sizeof(struct imapurl));
     imapurl.server = config_servername;
-    imapurl.mailbox = extname;
-    imapurl.user = userbuf;
     imapurl.uidvalidity = mailbox->i.uidvalidity;
+
+    assert(namespace.mboxname_toexternal != NULL);
+    (*namespace.mboxname_toexternal)(&namespace, mailbox->name, NULL, extname);
+    imapurl.mailbox = extname;
+
     if (event->type & (EVENT_MESSAGE_NEW|EVENT_MESSAGE_APPEND) && event->uidset) {
 	imapurl.uid = seqset_first(event->uidset);
 	/* don't add uidset parameter to MessageNew and MessageAppend events */
@@ -1021,6 +1002,7 @@ EXPORTED void mboxevent_extract_mailbox(struct mboxevent *event,
 	event->uidset = NULL;
     }
 
+    /* all events needs uri parameter */
     imapurl_toURL(url, &imapurl);
     FILL_STRING_PARAM(event, EVENT_URI, xstrdup(url));
 
@@ -1031,8 +1013,9 @@ EXPORTED void mboxevent_extract_mailbox(struct mboxevent *event,
 
     /* mailbox related events also require mailboxID */
     if (event->type & MAILBOX_EVENTS) {
-	FILL_STRING_PARAM(event, EVENT_MAILBOX_ID, xstrdup(url));
+	FILL_STRING_PARAM(event, EVENT_MAILBOX_ID, xstrdup(mailbox->uniqueid));
     }
+
     if (mboxevent_expected_param(event->type, EVENT_UIDNEXT)) {
 	FILL_UNSIGNED_PARAM(event, EVENT_UIDNEXT, mailbox->i.last_uid+1);
     }
@@ -1060,28 +1043,19 @@ void mboxevent_extract_old_mailbox(struct mboxevent *event,
     struct imapurl imapurl;
     char url[MAX_MAILBOX_PATH+1];
     char extname[MAX_MAILBOX_NAME];
-    char *userbuf;
 
     if (!event)
 	return;
 
-    /* translate internal mailbox name to external */
-    assert(namespace.mboxname_toexternal != NULL);
-    (*namespace.mboxname_toexternal)(&namespace, mailbox->name,
-				     mboxname_to_userid(mailbox->name), extname);
-
-    /* translate any separators in user */
-    userbuf = (char *)mboxname_to_userid(mailbox->name);
-    if (userbuf != NULL)
-	mboxname_hiersep_toexternal(&namespace, userbuf,
-				    config_virtdomains ? strcspn(userbuf, "@") : 0);
-
     memset(&imapurl, 0, sizeof(struct imapurl));
     imapurl.server = config_servername;
-    imapurl.mailbox = extname;
-    imapurl.user = userbuf;
     imapurl.uidvalidity = mailbox->i.uidvalidity;
 
+    /* translate internal mailbox name to external */
+    assert(namespace.mboxname_toexternal != NULL);
+    (*namespace.mboxname_toexternal)(&namespace, mailbox->name, NULL, extname);
+    imapurl.mailbox = extname;
+
     imapurl_toURL(url, &imapurl);
     FILL_STRING_PARAM(event, EVENT_OLD_MAILBOX_ID, xstrdup(url));
 }
