@@ -39,8 +39,6 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: sync_support.h,v 1.12 2010/01/06 17:01:42 murch Exp $
- *
  * Original version written by David Carter <dpc22@cam.ac.uk>
  * Rewritten and integrated into Cyrus by Ken Murchison <ken@oceana.com>
  */
@@ -56,8 +54,6 @@
 #define SYNC_MESSAGE_LIST_HASH_SIZE      (65536)
 #define SYNC_MESSAGE_LIST_MAX_OPEN_FILES (64)
 
-int sync_eatlines_unsolicited(struct protstream *pin, int c);
-
 void sync_printdate(struct protstream *out, time_t time);
 time_t sync_parsedate(const char *s);
 int sync_getflags(struct dlist *kl,
@@ -89,7 +85,7 @@ struct sync_msgid {
     struct sync_msgid *next;
     struct sync_msgid *hash_next;
     struct message_guid guid;
-    int mark;
+    int need_upload;
 };
 
 struct sync_msgid_list {
@@ -98,13 +94,13 @@ struct sync_msgid_list {
     struct sync_msgid **hash;
     int hash_size;
     int count;      /* Total number of messages in list    */
-    int marked;     /* Number of reserved messages in list */
+    int toupload;   /* Number of messages needing upload in list */
 };
 
 struct sync_msgid_list *sync_msgid_list_create(int hash_size);
 
-struct sync_msgid *sync_msgid_add(struct sync_msgid_list *list,
-				  struct message_guid *guid);
+struct sync_msgid *sync_msgid_insert(struct sync_msgid_list *list,
+				     struct message_guid *guid);
 
 void sync_msgid_remove(struct sync_msgid_list *l,
 		       struct message_guid *guid);
@@ -139,18 +135,21 @@ struct sync_folder {
     struct sync_folder *next;
     char *uniqueid;
     char *name;
+    uint32_t mbtype;
     char *part;
     char *acl;
-    unsigned last_uid;
+    uint32_t last_uid;
     modseq_t highestmodseq;
-    unsigned options;
-    unsigned long uidvalidity;
-    bit32 sync_crc;
-    unsigned long recentuid;
+    uint32_t options;
+    uint32_t uidvalidity;
+    struct synccrcs synccrcs;
+    uint32_t recentuid;
     time_t recenttime;
     time_t pop3_last_login;
+    time_t pop3_show_after;
+    struct sync_annot_list *annots;
     struct quota quota;
-    int   mark; 
+    int   mark;
     int   reserve;  /* Folder has been processed by reserve operation */
 };
 
@@ -163,15 +162,18 @@ struct sync_folder_list *sync_folder_list_create(void);
 
 struct sync_folder *sync_folder_list_add(struct sync_folder_list *l,
 					 const char *uniqueid, const char *name,
-					 const char *part, const char *acl, 
+					 uint32_t mbtype,
+					 const char *part, const char *acl,
 					 uint32_t options,
-					 uint32_t uidvalidity, 
+					 uint32_t uidvalidity,
 					 uint32_t last_uid,
 					 modseq_t highestmodseq,
-					 uint32_t crc,
+					 struct synccrcs synccrcs,
 					 uint32_t recentuid,
 					 time_t recenttime,
-					 time_t pop3_last_login);
+					 time_t pop3_last_login,
+					 time_t pop3_show_after,
+					 struct sync_annot_list *annots);
 
 struct sync_folder *sync_folder_lookup(struct sync_folder_list *l,
 				       const char *uniqueid);
@@ -191,6 +193,7 @@ struct sync_rename {
     char *oldname;
     char *newname;
     char *part;
+    unsigned uidvalidity;
     int   done;
 };
 
@@ -204,8 +207,9 @@ struct sync_rename_list *sync_rename_list_create(void);
 
 
 struct sync_rename *sync_rename_list_add(struct sync_rename_list *l,
-					      const char *id, const char *oldname,
-					      const char *newname, const char *partition);
+					 const char *id, const char *oldname,
+					 const char *newname, const char *partition,
+					 unsigned uidvalidity);
 
 struct sync_rename *sync_rename_lookup(struct sync_rename_list *l,
 					    const char *oldname);
@@ -217,7 +221,7 @@ void sync_rename_list_free(struct sync_rename_list **lp);
 struct sync_quota {
     struct sync_quota *next;
     char *root;
-    int limit;
+    quota_t limits[QUOTA_NUMRESOURCES];
     int done;
 };
 
@@ -230,13 +234,16 @@ struct sync_quota_list {
 struct sync_quota_list *sync_quota_list_create(void);
 
 struct sync_quota *sync_quota_list_add(struct sync_quota_list *l,
-					    const char *root, int limit);
+				       const char *root);
 
 struct sync_quota *sync_quota_lookup(struct sync_quota_list *l,
 					  const char *name);
 
 void sync_quota_list_free(struct sync_quota_list **lp);
 
+void sync_encode_quota_limits(struct dlist *kl, const quota_t limits[QUOTA_NUMRESOURCES]);
+void sync_decode_quota_limits(/*const*/ struct dlist *kl, quota_t limits[QUOTA_NUMRESOURCES]);
+
 /* ====================================================================== */
 
 struct sync_name {
@@ -295,6 +302,7 @@ struct sync_sieve {
     char *user;
     char *name;
     time_t last_update;
+    struct message_guid guid;
     int active;
     int mark;
 };
@@ -308,12 +316,13 @@ struct sync_sieve_list {
 struct sync_sieve_list *sync_sieve_list_create(void);
 
 void sync_sieve_list_add(struct sync_sieve_list *l,
-			 const char *name, time_t last_update, int active);
+			 const char *name, time_t last_update, 
+			 struct message_guid *guidp, int active);
 
 struct sync_sieve *sync_sieve_lookup(struct sync_sieve_list *l,
-					  char *name);
+				     const char *name);
 
-void sync_sieve_list_set_active(struct sync_sieve_list *l, char *name);
+void sync_sieve_list_set_active(struct sync_sieve_list *l, const char *name);
 
 void sync_sieve_list_free(struct sync_sieve_list **lp);
 
@@ -336,7 +345,7 @@ struct sync_annot {
     struct sync_annot *next;
     char *entry;
     char *userid;
-    char *value;
+    struct buf value;
     int mark;
 };
 
@@ -350,7 +359,7 @@ struct sync_annot_list *sync_annot_list_create(void);
 
 void sync_annot_list_add(struct sync_annot_list *l,
 			 const char *entry, const char *userid,
-			 const char *value);
+			 const struct buf *value);
 
 void sync_annot_list_free(struct sync_annot_list **lp);
 
@@ -370,8 +379,8 @@ struct sync_action_list {
 
 struct sync_action_list *sync_action_list_create(void);
 
-void sync_action_list_add(struct sync_action_list *l, char *name,
-			  char *user);
+void sync_action_list_add(struct sync_action_list *l,
+		          const char *name, const char *user);
 
 void sync_action_list_free(struct sync_action_list **lp);
 
@@ -380,13 +389,16 @@ void sync_action_list_free(struct sync_action_list **lp);
 void sync_send_response(struct dlist *kl, struct protstream *out);
 void sync_send_apply(struct dlist *kl, struct protstream *out);
 void sync_send_lookup(struct dlist *kl, struct protstream *out);
+void sync_send_set(struct dlist *kl, struct protstream *out);
 
 struct dlist *sync_parseline(struct protstream *in);
 
 /* ====================================================================== */
 
 int addmbox(char *name, int matchlen, int maycreate, void *rock);
-int addmbox_sub(char *name, int matchlen, int maycreate, void *rock);
+int addmbox_sub(void *rockp, const char *key, size_t keylen,
+		const char *data __attribute__((unused)),
+		size_t datalen __attribute__((unused)));
 
 int sync_mailbox(struct mailbox *mailbox,
 		 struct sync_folder *remote,
@@ -395,9 +407,30 @@ int sync_mailbox(struct mailbox *mailbox,
 		 int printrecords);
 
 int parse_upload(struct dlist *kr, struct mailbox *mailbox,
-		 struct index_record *record);
+		 struct index_record *record,
+		 struct sync_annot_list **annotsp);
 int sync_append_copyfile(struct mailbox *mailbox,
-			 struct index_record *record);
+			 struct index_record *record,
+			 const struct sync_annot_list *sal);
+
+/* ====================================================================== */
+
+int read_annotations(const struct mailbox *,
+		     const struct index_record *,
+		     struct sync_annot_list **);
+void encode_annotations(struct dlist *parent,
+			struct index_record *record,
+			const struct sync_annot_list *);
+int decode_annotations(/*const*/struct dlist *,
+		       struct sync_annot_list **,
+		       struct index_record *);
+int apply_annotations(struct mailbox *mailbox,
+		      const struct index_record *record,
+		      const struct sync_annot_list *local_annots,
+		      const struct sync_annot_list *remote_annots,
+		      int local_wins);
+int diff_annotations(const struct sync_annot_list *local_annots,
+		     const struct sync_annot_list *remote_annots);
 
 /* ====================================================================== */
 
