@@ -42,8 +42,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: tls.c,v 1.67 2010/01/06 17:01:42 murch Exp $
  */
 
 /*
@@ -123,22 +121,18 @@
 #include "assert.h"
 #include "nonblock.h"
 #include "xmalloc.h"
-#include "xstrlcat.h"
-#include "xstrlcpy.h"
 #include "tls.h"
+#include "tls_th-lock.h"
 
 /* Session caching/reuse stuff */
 #include "global.h"
 #include "cyrusdb.h"
 
-#define DB (config_tlscache_db) /* sessions are binary -> MUST use DB3 */
+#define DB (config_tls_sessions_db) /* sessions are binary -> MUST use DB3 */
 
 static struct db *sessdb = NULL;
 static int sess_dbopen = 0;
 
-/* We must keep some of the info available */
-static const char hexcodes[] = "0123456789ABCDEF";
-
 enum {
     var_imapd_tls_loglevel = 0,
     var_proxy_tls_loglevel = 0,
@@ -155,16 +149,19 @@ static int tls_clientengine = 0; /* client engine initialized? */
 static int do_dump = 0;		/* actively dumping protocol? */
 
 
-int tls_enabled(void)
+EXPORTED int tls_enabled(void)
 {
     const char *val;
 
-    val = config_getstring(IMAPOPT_TLS_CERT_FILE);
+    val = config_getstring(IMAPOPT_TLS_SERVER_CERT);
     if (!val || !strcasecmp(val, "disabled")) return 0;
 
-    val = config_getstring(IMAPOPT_TLS_KEY_FILE);
+    val = config_getstring(IMAPOPT_TLS_SERVER_KEY);
     if (!val || !strcasecmp(val, "disabled")) return 0;
 
+    if (config_getswitch(IMAPOPT_CHATTY))
+	    syslog(LOG_INFO, "TLS is available.");
+
     return 1;
 }
 
@@ -172,7 +169,7 @@ int tls_enabled(void)
  * tim - this seems to just be giving logging messages
  */
 
-static void apps_ssl_info_callback(SSL * s, int where, int ret)
+static void apps_ssl_info_callback(const SSL * s, int where, int ret)
 {
     char   *str;
     int     w;
@@ -209,7 +206,6 @@ static void apps_ssl_info_callback(SSL * s, int where, int ret)
     }
 }
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
 /* taken from OpenSSL apps/s_cb.c
    not thread safe! */
 static RSA *tmp_rsa_cb(SSL * s __attribute__((unused)),
@@ -223,51 +219,26 @@ static RSA *tmp_rsa_cb(SSL * s __attribute__((unused)),
     }
     return (rsa_tmp);
 }
-#endif
-
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-/* replacements for new 1.1 API accessors */
-/* XXX probably put these somewhere central */
-static int DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g)
-{
-    if (p == NULL || g == NULL) return 0;
-    dh->p = p;
-    dh->q = q; /* optional */
-    dh->g = g;
-    return 1;
-}
-#endif
 
 #if (OPENSSL_VERSION_NUMBER >= 0x0090800fL)
 /* Logic copied from OpenSSL apps/s_server.c: give the TLS context
  * DH params to work with DHE-* cipher suites. Hardcoded fallback
- * in case no DH params in tls_key_file or tls_cert_file.
- * Modified quite a bit for openssl 1.1.0 compatibility.
- * XXX we might be able to just replace this with DH_get_1024_160?
- * XXX the apps/s_server.c example doesn't use this anymore at all.
+ * in case no DH params in server_key or server_cert.
  */
 static DH *get_dh1024(void)
 {
     /* Second Oakley group 1024-bits MODP group from RFC2409 */
-    DH *dh;
-    BIGNUM *p = NULL, *g = NULL;
+    DH *dh=NULL;
 
-    dh = DH_new();
-    if (!dh) return NULL;
+    if ((dh=DH_new()) == NULL) return(NULL);
+    dh->p=get_rfc2409_prime_1024(NULL);
+    dh->g=NULL;
+    BN_dec2bn(&(dh->g), "2");
+    if ((dh->p == NULL) || (dh->g == NULL)) return(NULL);
 
-    p = get_rfc2409_prime_1024(NULL);
-    BN_dec2bn(&g, "2");
-
-    if (DH_set0_pqg(dh, p, NULL, g))
-	return dh;
-
-    if (g) BN_free(g);
-    if (p) BN_free(p);
-    DH_free(dh);
-
-    return NULL;
+    return(dh);
+	
 }
-
 static DH *load_dh_param(const char *keyfile, const char *certfile)
 {
     DH *ret=NULL;
@@ -281,9 +252,9 @@ static DH *load_dh_param(const char *keyfile, const char *certfile)
 
     if (ret == NULL) {
 	ret = get_dh1024();
-	syslog(LOG_NOTICE, "imapd:Loading hard-coded DH parameters");
+	syslog(LOG_NOTICE, "inittls: Loading hard-coded DH parameters");
     } else {
-	syslog(LOG_NOTICE, "imapd:Loading DH parameters from file");
+	syslog(LOG_NOTICE, "inittls: Loading DH parameters from file");
     }
 
     if (bio != NULL) BIO_free(bio);
@@ -321,9 +292,9 @@ static int verify_callback(int ok, X509_STORE_CTX * ctx)
 	    verify_error = X509_V_ERR_CERT_CHAIN_TOO_LONG;
 	}
     }
-    switch (err) {
+    switch (ctx->error) {
     case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
-	X509_NAME_oneline(X509_get_issuer_name(err_cert), buf, sizeof(buf));
+	X509_NAME_oneline(X509_get_issuer_name(ctx->current_cert), buf, sizeof(buf));
 	syslog(LOG_NOTICE, "issuer= %s", buf);
 	break;
     case X509_V_ERR_CERT_NOT_YET_VALID:
@@ -340,6 +311,22 @@ static int verify_callback(int ok, X509_STORE_CTX * ctx)
 }
 
 
+#if (OPENSSL_VERSION_NUMBER >= 0x0090806fL)
+static int servername_callback(SSL *ssl, int *ad __attribute__((unused)),
+			       void *arg __attribute__((unused)))
+{
+    const char *servername = SSL_get_servername(ssl, TLSEXT_NAMETYPE_host_name);
+
+    if (servername) {
+	syslog(LOG_DEBUG, "TLS Server Name Indication (SNI) Extension: \"%s\"",
+	       servername);
+    }
+
+    return SSL_TLSEXT_ERR_OK;
+}
+#endif
+
+
 /*
  * taken from OpenSSL crypto/bio/b_dump.c, modified to save a lot of strcpy
  * and strcat by Matti Aarnio.
@@ -450,8 +437,13 @@ static int set_cert_stuff(SSL_CTX * ctx,
 	/* Now we know that a key and cert have been set against
          * the SSL context */
 	if (!SSL_CTX_check_private_key(ctx)) {
-	    syslog(LOG_ERR,
-		   "Private key does not match the certificate public key");
+	    syslog(
+		    LOG_ERR,
+		    "Private key '%s' does not match public key '%s'",
+		    cert_file,
+		    key_file
+		);
+
 	    return (0);
 	}
     }
@@ -471,8 +463,6 @@ static int new_session_cb(SSL *ssl __attribute__((unused)),
     time_t expire;
     int ret = -1;
     unsigned char *asn;
-    const unsigned char *session_id = NULL;
-    unsigned int session_id_length = 0;
 
     assert(sess);
 
@@ -498,11 +488,9 @@ static int new_session_cb(SSL *ssl __attribute__((unused)),
 
     if (len) {
 	/* store the session in our database */
-
-	session_id = SSL_SESSION_get_id(sess, &session_id_length);
 	do {
-	    ret = DB->store(sessdb, (const char *) session_id,
-			    session_id_length,
+	    ret = cyrusdb_store(sessdb, (const char *) sess->session_id,
+			    sess->session_id_length,
 			    (const char *) data, len + sizeof(time_t), NULL);
 	} while (ret == CYRUSDB_AGAIN);
     }
@@ -513,8 +501,8 @@ static int new_session_cb(SSL *ssl __attribute__((unused)),
     if (var_imapd_tls_loglevel > 0) {
 	unsigned int i;
 	char idstr[SSL_MAX_SSL_SESSION_ID_LENGTH*2 + 1];
-	for (i = 0; i < session_id_length; i++) {
-	    sprintf(idstr+i*2, "%02X", session_id[i]);
+	for (i = 0; i < sess->session_id_length; i++) {
+	    sprintf(idstr+i*2, "%02X", sess->session_id[i]);
 	}
 	syslog(LOG_DEBUG, "new TLS session: id=%s, expire=%s, status=%s",
 	       idstr, ctime(&expire), ret ? "failed" : "ok");
@@ -526,7 +514,7 @@ static int new_session_cb(SSL *ssl __attribute__((unused)),
 /*
  * Function for removing a session from our database.
  */
-static void remove_session(const unsigned char *id, int idlen)
+static void remove_session(unsigned char *id, int idlen)
 {
     int ret;
 
@@ -536,18 +524,14 @@ static void remove_session(const unsigned char *id, int idlen)
     if (!sess_dbopen) return;
 
     do {
-	ret = DB->delete(sessdb, (const char *) id, idlen, NULL, 1);
+	ret = cyrusdb_delete(sessdb, (const char *) id, idlen, NULL, 1);
     } while (ret == CYRUSDB_AGAIN);
 
     /* log this transaction */
     if (var_imapd_tls_loglevel > 0) {
-	int i;
 	char idstr[SSL_MAX_SSL_SESSION_ID_LENGTH*2 + 1];
 
-	for (i = 0; i < idlen; i++) {
-	    sprintf(idstr+i*2, "%02X", id[i]);
-	}
-	    
+	bin_to_hex(id, idlen, idstr, BH_UPPER);
 	syslog(LOG_DEBUG, "remove TLS session: id=%s", idstr);
     }
 }
@@ -561,14 +545,9 @@ static void remove_session(const unsigned char *id, int idlen)
 static void remove_session_cb(SSL_CTX *ctx __attribute__((unused)),
 			      SSL_SESSION *sess)
 {
-    const unsigned char *session_id = NULL;
-    unsigned int session_id_length = 0;
-
     assert(sess);
 
-    session_id = SSL_SESSION_get_id(sess, &session_id_length);
-
-    remove_session(session_id, session_id_length);
+    remove_session(sess->session_id, sess->session_id_length);
 }
 
 /*
@@ -577,17 +556,12 @@ static void remove_session_cb(SSL_CTX *ctx __attribute__((unused)),
  * called, also when session caching was disabled.  We lookup the
  * session in our database in case it was stored by another process.
  */
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-static SSL_SESSION *get_session_cb(SSL *ssl __attribute__((unused)),
-				   const unsigned char *id, int idlen, int *copy)
-#else
 static SSL_SESSION *get_session_cb(SSL *ssl __attribute__((unused)),
 				   unsigned char *id, int idlen, int *copy)
-#endif
 {
     int ret;
     const char *data = NULL;
-    int len = 0;
+    size_t len = 0;
     time_t expire = 0, now = time(0);
     SSL_SESSION *sess = NULL;
 
@@ -597,7 +571,7 @@ static SSL_SESSION *get_session_cb(SSL *ssl __attribute__((unused)),
     if (!sess_dbopen) return NULL;
 
     do {
-	ret = DB->fetch(sessdb, (const char *) id, idlen, &data, &len, NULL);
+	ret = cyrusdb_fetch(sessdb, (const char *) id, idlen, &data, &len, NULL);
     } while (ret == CYRUSDB_AGAIN);
 
     if (!ret && data) {
@@ -658,7 +632,8 @@ static int tls_rand_init(void)
   */
 
 /* must be called after cyrus_init */
-int     tls_init_serverengine(const char *ident,
+// I am the server
+EXPORTED int     tls_init_serverengine(const char *ident,
 			      int verifydepth,
 			      int askcert,
 			      int tlsonly)
@@ -666,13 +641,22 @@ int     tls_init_serverengine(const char *ident,
     int     off = 0;
     int     verify_flags = SSL_VERIFY_NONE;
     const char   *cipher_list;
-    const char   *CApath;
-    const char   *CAfile;
-    const char   *s_cert_file;
-    const char   *s_key_file;
-    int    requirecert;
-    int    server_cipher_order;
-    int    timeout;
+    const char   *client_ca_dir;
+    const char   *client_ca_file;
+    const char   *server_ca_file;
+    const char   *server_cert_file;
+    const char   *server_key_file;
+    enum enum_value tls_client_certs;
+    int server_cipher_order;
+    int timeout;
+
+    /* Whether or not to use any client certificate CA context to
+     * verify client SSL certificates with.
+     *
+     * This is initially switched off by a value of "off" for the
+     * tls_client_certs setting, or should no client_ca_dir nor
+     * client_ca_file be specified. */
+    int use_client_certs = 1;
 
     if (tls_serverengine)
 	return (0);				/* already running */
@@ -689,7 +673,6 @@ int     tls_init_serverengine(const char *ident,
 
     /* even if we want TLS only, we use SSLv23 server method so we can
        deal with a client sending an SSLv2 greeting message */
-
     s_ctx = SSL_CTX_new(SSLv23_server_method());
     if (s_ctx == NULL) {
 	return (-1);
@@ -699,71 +682,177 @@ int     tls_init_serverengine(const char *ident,
 
     const char *tls_versions = config_getstring(IMAPOPT_TLS_VERSIONS);
 
-    off |= SSL_OP_NO_SSLv2;
-    off |= SSL_OP_NO_SSLv3;
+    if (strstr(tls_versions, "ssl2") == NULL || tlsonly) {
+	//syslog(LOG_DEBUG, "TLS server engine: Disabled SSLv2");
+	off |= SSL_OP_NO_SSLv2;
+    }
+
+    if (strstr(tls_versions, "ssl3") == NULL || tlsonly) {
+	//syslog(LOG_DEBUG, "TLS server engine: Disabled SSLv3");
+	off |= SSL_OP_NO_SSLv3;
+    }
 
     if (strstr(tls_versions, "tls1_2") == NULL) {
 #if (OPENSSL_VERSION_NUMBER >= 0x1000105fL)
+	//syslog(LOG_DEBUG, "TLS server engine: Disabled TLSv1.2");
 	off |= SSL_OP_NO_TLSv1_2;
 #else
 	syslog(LOG_ERR, "ERROR: TLSv1.2 configured, OpenSSL < 1.0.1e insufficient");
-#endif
+#endif // (OPENSSL_VERSION_NUMBER >= 0x1000105fL)
     }
 
     if (strstr(tls_versions, "tls1_1") == NULL) {
 #if (OPENSSL_VERSION_NUMBER >= 0x1000000fL)
+	//syslog(LOG_DEBUG, "TLS server engine: Disabled TLSv1.1");
 	off |= SSL_OP_NO_TLSv1_1;
 #else
 	syslog(LOG_ERR, "ERROR: TLSv1.1 configured, OpenSSL < 1.0.0 insufficient");
-#endif
+#endif // (OPENSSL_VERSION_NUMBER >= 0x1000000fL)
     }
+
     if (strstr(tls_versions, "tls1_0") == NULL) {
+	//syslog(LOG_DEBUG, "TLS server engine: Disabled TLSv1.0");
 	off |= SSL_OP_NO_TLSv1;
     }
 
     server_cipher_order = config_getswitch(IMAPOPT_TLS_PREFER_SERVER_CIPHERS);
     if (server_cipher_order)
-        off |= SSL_OP_CIPHER_SERVER_PREFERENCE;
+	off |= SSL_OP_CIPHER_SERVER_PREFERENCE;
 
+#if (OPENSSL_VERSION_NUMBER >= 0x1000000fL)
+    if (!config_getswitch(IMAPOPT_TLS_COMPRESSION)) {
+	off |= SSL_OP_NO_COMPRESSION;
+	syslog(LOG_DEBUG, "TLS client engine: Setting SSL_OP_NO_COMPRESSION");
+    }
+#endif // (OPENSSL_VERSION_NUMBER >= 0x1000000fL)
+    
     SSL_CTX_set_options(s_ctx, off);
-    SSL_CTX_set_info_callback(s_ctx, (void (*)()) apps_ssl_info_callback);
+    SSL_CTX_set_info_callback(s_ctx, apps_ssl_info_callback);
 
-    cipher_list = config_getstring(IMAPOPT_TLS_CIPHER_LIST);
+    cipher_list = config_getstring(IMAPOPT_TLS_CIPHERS);
     if (!SSL_CTX_set_cipher_list(s_ctx, cipher_list)) {
-	syslog(LOG_ERR,"TLS server engine: cannot load cipher list '%s'",
-	       cipher_list);
+	syslog(
+		LOG_ERR,
+		"TLS server engine: cannot load cipher list '%s'",
+		cipher_list
+	    );
+
 	return (-1);
     }
 
-    CAfile = config_getstring(IMAPOPT_TLS_CA_FILE);
-    CApath = config_getstring(IMAPOPT_TLS_CA_PATH);
+    tls_client_certs = config_getenum(IMAPOPT_TLS_CLIENT_CERTS);
 
-    if (CAfile || CApath) {
-	if ((!SSL_CTX_load_verify_locations(s_ctx, CAfile, CApath)) ||
-	    (!SSL_CTX_set_default_verify_paths(s_ctx))) {
-	    /* just a warning since this is only necessary for client auth */
-	    syslog(LOG_NOTICE,"TLS server engine: cannot load CA data. Disabling client certs.");
-	    askcert = 0;
+    if (tls_client_certs != IMAP_ENUM_TLS_CLIENT_CERTS_OFF) {
+	// The use of client certificates for authentication is
+	// optional or required.
+	client_ca_dir  = config_getstring(IMAPOPT_TLS_CLIENT_CA_DIR);
+	client_ca_file = config_getstring(IMAPOPT_TLS_CLIENT_CA_FILE);
+
+	    if (config_debug) {
+		    syslog(
+			LOG_DEBUG, "tls_client_ca_dir=%s tls_client_ca_file=%s",
+			IS_NULL(client_ca_dir), IS_NULL(client_ca_file)
+		    );
+	    }
+
+	if (client_ca_dir || client_ca_file) {
+	    // Attempt to load client_ca_dir and/or client_ca_file
+	    if ((!SSL_CTX_load_verify_locations(s_ctx, client_ca_file, client_ca_dir)) ||
+		(!SSL_CTX_set_default_verify_paths(s_ctx))) {
+
+		// Just a warning since this is only necessary for client auth.
+		syslog(
+			LOG_WARNING,
+			"TLS server engine: Failed loading client CA data, cert auth disabled."
+		    );
+
+		use_client_certs = 0;
+	    }
+	} else { // (client_ca_dir || client_ca_file)
+	    // We have no CA to verify client certificates against.
+	    syslog(LOG_DEBUG, "TLS server engine: No client CA data configured.");
+
+	    // This is fatal should client certificates be required.
+	    if (tls_client_certs == IMAP_ENUM_TLS_CLIENT_CERTS_REQUIRE) {
+		return (-1);
+	    }
+
+	    use_client_certs = 0;
 	}
-    } else {
-	askcert = 0;
+
+    } else { // (tls_client_certs != IMAP_ENUM_TLS_CLIENT_CERTS_OFF)
+	// The use of client certificates is turned off.
+	use_client_certs = 0;
     }
 
-    s_cert_file = config_getstring(IMAPOPT_TLS_CERT_FILE);
-    s_key_file = config_getstring(IMAPOPT_TLS_KEY_FILE);
+    server_ca_file = config_getstring(IMAPOPT_TLS_SERVER_CA_FILE);
+    server_cert_file = config_getstring(IMAPOPT_TLS_SERVER_CERT);
+    server_key_file = config_getstring(IMAPOPT_TLS_SERVER_KEY);
 
-    if (!set_cert_stuff(s_ctx, s_cert_file, s_key_file)) {
-	syslog(LOG_ERR,"TLS server engine: cannot load cert/key data");
+    if (config_debug) {
+	    syslog(
+		LOG_DEBUG, "tls_server_cert=%s tls_server_key=%s",
+		IS_NULL(server_cert_file), IS_NULL(server_key_file)
+	    );
+    }
+
+    /* Only consider adding additional CA certificates -used to verify certificates
+     * issued to clients, which may have been issued by an intermediate CA and
+     * not the CA that issued our server certificate- when indeed we are using
+     * SSL/TLS on this server.
+     */
+    if (server_ca_file && !strcasecmp(server_cert_file, "disabled")) {
+	SSL_CTX_set_mode(s_ctx,SSL_CTX_get_mode(s_ctx) | SSL_MODE_NO_AUTO_CHAIN);
+	BIO *filebio = BIO_new_file(server_ca_file, "r");
+
+	if (filebio) {
+	    X509 *cacert;
+
+	    syslog(
+		    LOG_DEBUG,
+		    "TLS server engine: loading additional client CA information from file %s",
+		    server_ca_file
+		);
+
+	    while ((cacert = PEM_read_bio_X509(filebio,NULL,NULL,NULL)) != NULL) {
+		char buf[256];
+		X509_NAME_oneline(X509_get_subject_name(cacert), buf, sizeof(buf));
+
+		if (!SSL_CTX_add_extra_chain_cert(s_ctx, cacert)) {
+		    syslog(
+			    LOG_ERR,
+			    "TLS server engine: failed to add client_ca_cert to chain: %s",
+			    buf
+			);
+
+		    X509_free(cacert);
+		} else {
+		    syslog(LOG_DEBUG, "TLS server engine: added CA cert to chain: %s", buf);
+		}
+	    }
+
+	    BIO_free(filebio);
+	} else {
+	    syslog(
+		    LOG_ERR,
+		    "TLS server engine: Cannot load additional client CA information from file %s",
+		    server_ca_file
+		);
+
+	    return (-1);
+	}
+    }
+
+    if (!set_cert_stuff(s_ctx, server_cert_file, server_key_file)) {
+	syslog(LOG_ERR, "TLS server engine: cannot load server cert/key data.");
 	return (-1);
     }
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
     SSL_CTX_set_tmp_rsa_callback(s_ctx, tmp_rsa_cb);
-#endif
 
 #if (OPENSSL_VERSION_NUMBER >= 0x0090800fL)
     /* Load DH params for DHE-* key exchanges */
-    SSL_CTX_set_tmp_dh(s_ctx, load_dh_param(s_key_file, s_cert_file));
+    SSL_CTX_set_tmp_dh(s_ctx, load_dh_param(server_key_file, server_cert_file));
 #endif
 
 #if (OPENSSL_VERSION_NUMBER >= 0x1000103fL)
@@ -780,24 +869,43 @@ int     tls_init_serverengine(const char *ident,
 #endif
 
     verify_depth = verifydepth;
-    if (askcert != 0)
-	verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE;
 
-    requirecert = config_getswitch(IMAPOPT_TLS_REQUIRE_CERT);
-    if (requirecert)
-	verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT
-	    | SSL_VERIFY_CLIENT_ONCE;
+    if (!use_client_certs) {
+	if ((tls_client_certs == IMAP_ENUM_TLS_CLIENT_CERTS_REQUIRE) && askcert)
+	    syslog(LOG_ERR, "TLS server engine: No client cert CA specified.");
+
+    } else {
+	STACK_OF(X509_NAME) *CAnames = SSL_load_client_CA_file(client_ca_file);
+
+	if (!CAnames || sk_X509_NAME_num(CAnames) < 1) {
+	    syslog(
+		    LOG_ERR,
+		    "TLS server engine: No client CA certs specified. Client side certs may not work"
+		);
+
+	} else {
+	    if (askcert || tls_client_certs)
+		verify_flags = SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE;
+
+	    if (tls_client_certs == IMAP_ENUM_TLS_CLIENT_CERTS_REQUIRE)
+		verify_flags |= SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
+
+	    SSL_CTX_set_client_CA_list(s_ctx, CAnames);
+
+	    syslog(
+		    LOG_DEBUG,
+		    "Set client CA list: Client cert %srequested, %srequired",
+		    verify_flags & SSL_VERIFY_PEER ? "" : "not ",
+		    verify_flags & SSL_VERIFY_FAIL_IF_NO_PEER_CERT ? "":"not "
+		);
+	}
+    }
+
     SSL_CTX_set_verify(s_ctx, verify_flags, verify_callback);
 
-    if (askcert || requirecert) {
-      if (CAfile == NULL) {
-	  syslog(LOG_ERR, 
-		 "TLS server engine: No CA file specified. "
-		 "Client side certs will not work.");
-      } else {
-	  SSL_CTX_set_client_CA_list(s_ctx, SSL_load_client_CA_file(CAfile));
-      }
-    }
+#if (OPENSSL_VERSION_NUMBER >= 0x0090806fL)
+    SSL_CTX_set_tlsext_servername_callback(s_ctx, servername_callback);
+#endif
 
     /* Don't use an internal session cache */
     SSL_CTX_sess_set_cache_size(s_ctx, 1);  /* 0 is unlimited, so use 1 */
@@ -827,7 +935,7 @@ int     tls_init_serverengine(const char *ident,
 	SSL_CTX_sess_set_remove_cb(s_ctx, remove_session_cb);
 	SSL_CTX_sess_set_get_cb(s_ctx, get_session_cb);
 
-	fname = config_getstring(IMAPOPT_TLSCACHE_DB_PATH);
+	fname = config_getstring(IMAPOPT_TLS_SESSIONS_DB_PATH);
 
 	/* create the name of the db file */
 	if (!fname) {
@@ -835,7 +943,7 @@ int     tls_init_serverengine(const char *ident,
 	    fname = tofree;
 	}
 
-	r = (DB->open)(fname, CYRUSDB_CREATE, &sessdb);
+	r = cyrusdb_open(DB, fname, CYRUSDB_CREATE, &sessdb);
 	if (r != 0) {
 	    syslog(LOG_ERR, "DBERROR: opening %s: %s",
 		   fname, cyrusdb_strerror(ret));
@@ -887,11 +995,10 @@ static long bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi,
   * filled in if the client authenticated. 'ret' is the SSL connection
   * on success.
   */
-int tls_start_servertls(int readfd, int writefd, int timeout,
+EXPORTED int tls_start_servertls(int readfd, int writefd, int timeout,
 			int *layerbits, char **authid, SSL **ret)
 {
     int     sts;
-    int     j;
     unsigned int n;
     const SSL_CIPHER *cipher;
     X509   *peer;
@@ -1043,16 +1150,7 @@ int tls_start_servertls(int readfd, int writefd, int timeout,
 	    syslog(LOG_DEBUG, "issuer=%s", peer_issuer);
 
 	if (X509_digest(peer, EVP_md5(), md, &n)) {
-	    for (j = 0; j < (int) n; j++)
-	    {
-		fingerprint[j * 3] = hexcodes[(md[j] & 0xf0) >> 4];
-		fingerprint[(j * 3) + 1] = hexcodes[(md[j] & 0x0f)];
-		if (j + 1 != (int) n) {
-		    fingerprint[(j * 3) + 2] = '_';
-		} else {
-		    fingerprint[(j * 3) + 2] = '\0';
-		}
-	    }
+	    bin_to_hex(md, n, fingerprint, BH_UPPER|BH_SEPARATOR('_'));
 	    if (var_imapd_tls_loglevel >= 2)
 		syslog(LOG_DEBUG, "fingerprint=%s", fingerprint);
 	}
@@ -1082,18 +1180,30 @@ int tls_start_servertls(int readfd, int writefd, int timeout,
     }
 
     if (authid && *authid) {
-	syslog(LOG_NOTICE, "starttls: %s with cipher %s (%d/%d bits %s)"
-	                   " authenticated as %s", 
-	       tls_protocol, tls_cipher_name,
-	       tls_cipher_usebits, tls_cipher_algbits, 
-	       SSL_session_reused(tls_conn) ? "reused" : "new",
-	       *authid);
+	syslog(
+		LOG_NOTICE,
+		"starttls: %s with cipher %s (%d/%d bits %s) "
+		"authenticated as %s",
+		tls_protocol,
+		tls_cipher_name,
+		tls_cipher_usebits,
+		tls_cipher_algbits, 
+		SSL_session_reused(tls_conn) ? "reused" : "new",
+		*authid
+	    );
+
     } else {
-	syslog(LOG_NOTICE, "starttls: %s with cipher %s (%d/%d bits %s)"
-	                   " no authentication", 
-	       tls_protocol, tls_cipher_name,
-	       tls_cipher_usebits, tls_cipher_algbits,
-	       SSL_session_reused(tls_conn) ? "reused" : "new");
+	syslog(
+		LOG_NOTICE,
+		"starttls: %s with cipher %s (%d/%d bits %s) "
+		"no authentication",
+		tls_protocol,
+		tls_cipher_name,
+		tls_cipher_usebits,
+		tls_cipher_algbits,
+		SSL_session_reused(tls_conn) ? "reused" : "new"
+	    );
+
     }
 
  done:
@@ -1111,7 +1221,7 @@ int tls_start_servertls(int readfd, int writefd, int timeout,
     return r;
 }
 
-int tls_reset_servertls(SSL **conn)
+EXPORTED int tls_reset_servertls(SSL **conn)
 {
     int r = 0;
 
@@ -1138,12 +1248,12 @@ int tls_reset_servertls(SSL **conn)
     return r;
 }
 
-int tls_shutdown_serverengine(void)
+EXPORTED int tls_shutdown_serverengine(void)
 {
     int r;
 
     if (tls_serverengine && sess_dbopen) {
-	r = (DB->close)(sessdb);
+	r = cyrusdb_close(sessdb);
 	if (r) {
 	    syslog(LOG_ERR, "DBERROR: error closing tlsdb: %s",
 		   cyrusdb_strerror(r));
@@ -1164,8 +1274,8 @@ struct prunerock {
     int deletions;
 };
 
-static int prune_p(void *rock, const char *id, int idlen,
-		   const char *data, int datalen)
+static int prune_p(void *rock, const char *id, size_t idlen,
+		   const char *data, size_t datalen)
 {
     struct prunerock *prock = (struct prunerock *) rock;
     time_t expire;
@@ -1179,7 +1289,7 @@ static int prune_p(void *rock, const char *id, int idlen,
 
     /* log this transaction */
     if (var_imapd_tls_loglevel > 0) {
-	int i;
+	size_t i;
 	char idstr[SSL_MAX_SSL_SESSION_ID_LENGTH*2 + 1];
 
 	assert(idlen <= SSL_MAX_SSL_SESSION_ID_LENGTH);
@@ -1196,9 +1306,9 @@ static int prune_p(void *rock, const char *id, int idlen,
     return (expire < time(0));
 }
 
-static int prune_cb(void *rock, const char *id, int idlen,
+static int prune_cb(void *rock, const char *id, size_t idlen,
 		    const char *data __attribute__((unused)), 
-                    int datalen __attribute__((unused)))
+                    size_t datalen __attribute__((unused)))
 {
     struct prunerock *prock = (struct prunerock *) rock;
 
@@ -1210,14 +1320,14 @@ static int prune_cb(void *rock, const char *id, int idlen,
 }
 
 /* must be called after cyrus_init */
-int tls_prune_sessions(void)
+EXPORTED int tls_prune_sessions(void)
 {
     const char *fname = NULL;
     char *tofree = NULL;
     int ret;
     struct prunerock prock;
 
-    fname = config_getstring(IMAPOPT_TLSCACHE_DB_PATH);
+    fname = config_getstring(IMAPOPT_TLS_SESSIONS_DB_PATH);
 
    /* create the name of the db file */
     if (!fname) {
@@ -1225,7 +1335,7 @@ int tls_prune_sessions(void)
 	fname = tofree;
     }
 
-    ret = (DB->open)(fname, 0, &sessdb);
+    ret = cyrusdb_open(DB, fname, 0, &sessdb);
     if (ret != CYRUSDB_OK) {
 	syslog(LOG_ERR, "DBERROR: opening %s: %s",
 	       fname, cyrusdb_strerror(ret));
@@ -1235,8 +1345,8 @@ int tls_prune_sessions(void)
 	/* check each session in our database */
 	sess_dbopen = 1;
 	prock.count = prock.deletions = 0;
-	DB->foreach(sessdb, "", 0, &prune_p, &prune_cb, &prock, NULL);
-	(DB->close)(sessdb);
+	cyrusdb_foreach(sessdb, "", 0, &prune_p, &prune_cb, &prock, NULL);
+	cyrusdb_close(sessdb);
 	sessdb = NULL;
 	sess_dbopen = 0;
 
@@ -1250,7 +1360,7 @@ int tls_prune_sessions(void)
 }
 
 /* fill string buffer with info about tls connection */
-int tls_get_info(SSL *conn, char *buf, size_t len)
+EXPORTED int tls_get_info(SSL *conn, char *buf, size_t len)
 {
     int usebits = 0;
     int algbits = 0;
@@ -1264,16 +1374,17 @@ int tls_get_info(SSL *conn, char *buf, size_t len)
     return (strlen(buf));
 }
 
-int tls_init_clientengine(int verifydepth,
-			  char *var_tls_cert_file,
-			  char *var_tls_key_file)
+// I am the client
+HIDDEN int tls_init_clientengine(int verifydepth,
+			  const char *var_server_cert,
+			  const char *var_server_key)
 {
     int     off = 0;
     int     verify_flags = SSL_VERIFY_NONE;
-    const char   *CApath;
-    const char   *CAfile;
-    char   *c_cert_file;
-    char   *c_key_file;
+    const char   *server_ca_dir;
+    const char   *server_ca_file;
+    char   *client_cert;
+    char   *client_key;
     
     if (tls_clientengine)
 	return (0);				/* already running */
@@ -1287,47 +1398,60 @@ int tls_init_clientengine(int verifydepth,
 	printf("TLS client engine: cannot seed PRNG\n");
 	return -1;
     }
-    
+
+    /* XXX  May need to use only SSLv3 for iSchedule */
     c_ctx = SSL_CTX_new(SSLv23_client_method());
     if (c_ctx == NULL) {
 	return (-1);
     };
     
     off |= SSL_OP_ALL;		/* Work around all known bugs */
-    off |= SSL_OP_NO_SSLv2;
-    off |= SSL_OP_NO_SSLv3;
-
     SSL_CTX_set_options(c_ctx, off);
-    SSL_CTX_set_info_callback(c_ctx, (void (*)()) apps_ssl_info_callback);
-    
-    CAfile = config_getstring(IMAPOPT_TLS_CA_FILE);
-    CApath = config_getstring(IMAPOPT_TLS_CA_PATH);
+    SSL_CTX_set_info_callback(c_ctx, apps_ssl_info_callback);
 
-    if ((!SSL_CTX_load_verify_locations(c_ctx, CAfile, CApath)) ||
-	(!SSL_CTX_set_default_verify_paths(c_ctx))) {
-	/* just a warning since this is only necessary for client auth */
-	syslog(LOG_NOTICE,"TLS client engine: cannot load CA data");
+#if (OPENSSL_VERSION_NUMBER >= 0x1000000fL)
+    if (!config_getswitch(IMAPOPT_TLS_COMPRESSION)) {
+	off |= SSL_OP_NO_COMPRESSION;
+	syslog(LOG_DEBUG, "TLS client engine: Setting SSL_OP_NO_COMPRESSION");
+    }
+#endif // (OPENSSL_VERSION_NUMBER >= 0x1000000fL)
+    
+    server_ca_dir = config_getstring(IMAPOPT_TLS_SERVER_CA_DIR);
+    server_ca_file = config_getstring(IMAPOPT_TLS_SERVER_CA_FILE);
+
+    if (config_debug) {
+	    syslog(
+		LOG_DEBUG, "tls_server_ca_dir=%s tls_server_ca_file=%s",
+		IS_NULL(server_ca_dir), IS_NULL(server_ca_file)
+	    );
     }
 
-    if (strlen(var_tls_cert_file) == 0)
-	c_cert_file = NULL;
+    if (server_ca_dir || server_ca_file) {
+	if ((!SSL_CTX_load_verify_locations(c_ctx, server_ca_file, server_ca_dir)) ||
+	    (!SSL_CTX_set_default_verify_paths(c_ctx))) {
+	    /* just a warning since this is only necessary for client auth */
+	    syslog(LOG_NOTICE,"TLS client engine: cannot load CA data");
+	}
+    }
+
+    if (!var_server_cert || strlen(var_server_cert) == 0)
+	client_cert = NULL;
     else
-	c_cert_file = var_tls_cert_file;
-    if (strlen(var_tls_key_file) == 0)
-	c_key_file = NULL;
+	client_cert = xstrdup(var_server_cert);
+
+    if (!var_server_key || strlen(var_server_key) == 0)
+	client_key = NULL;
     else
-	c_key_file = var_tls_key_file;
+	client_key = xstrdup(var_server_key);
     
-    if (c_cert_file || c_key_file) {
-	if (!set_cert_stuff(c_ctx, c_cert_file, c_key_file)) {
-	    syslog(LOG_ERR,"TLS client engine: cannot load cert/key data");
+    if (client_cert || client_key) {
+	if (!set_cert_stuff(c_ctx, client_cert, client_key)) {
+	    syslog(LOG_ERR,"TLS client engine: cannot load client cert/key data");
 	    return (-1);
 	}
     }
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
     SSL_CTX_set_tmp_rsa_callback(c_ctx, tmp_rsa_cb);
-#endif
     
     verify_depth = verifydepth;
     SSL_CTX_set_verify(c_ctx, verify_flags, verify_callback);
@@ -1336,7 +1460,7 @@ int tls_init_clientengine(int verifydepth,
     return (0);
 }
 
-int tls_start_clienttls(int readfd, int writefd,
+HIDDEN int tls_start_clienttls(int readfd, int writefd,
 			int *layerbits, char **authid, SSL **ret,
 			SSL_SESSION **sess)
 {
@@ -1456,13 +1580,13 @@ int tls_start_clienttls(int readfd, int writefd,
 	tls_conn = NULL;
     }
     *ret = tls_conn;
-    return r;
 
+    return r;
 }
 
 #else
 
-int tls_enabled(void)
+EXPORTED int tls_enabled(void)
 {
     return 0;
 }
