@@ -45,6 +45,7 @@
 #include "ptloader.h"
 #include "exitcodes.h"
 #include "util.h"
+#include "assert.h"
 
 #ifdef HAVE_LDAP
 
@@ -629,58 +630,42 @@ static int ptsmodule_escape(
     return PTSM_OK;
 }
 
-static int ptsmodule_standard_root_dn(char *domain, const char **result)
+static int ptsmodule_standard_root_dn(const char *domain, const char **result)
 {
-    /* number of dots */
-    int dots;
-    /* the expected length of the result */
-    int root_dn_len;
+    const char *dc_sep = ",dc=";
+    char *domain_copy;
+    char *part, *tok_state;
+    struct buf buf = BUF_INITIALIZER;
 
-    char *buf;
-    char *part;
-    char *ptr;
+    assert(domain != NULL && domain[0] != '\0');
 
     syslog(LOG_DEBUG, "ptsmodule_standard_root_dn called for domain %s", domain);
 
-    for (dots = 0, buf=(char *)domain; *buf; buf++) {
-	if (*buf == '.') {
-	    dots++;
-	}
-    }
-
-    /* Each dot is to be replaced with ',dc=' (length 4), so add
-     * length 3 for each of them.
+    /* Each dot is to be replaced with ',dc='.
+     * We also need a leading 'dc=' at the start.
      */
-    root_dn_len = strlen(domain) + (dots * 3);
-
-    buf = xmalloc(root_dn_len);
-    buf[0] = '\0'; // (?)
-    /* AM: Above: Terminate the string by default, so strlen won't buffer-overflow later */
-
-    /* AM: No need for another allocation/dup */
-    part = strtok_r(domain, ".", &ptr);
+    domain_copy = xstrdup(domain);
+    part = strtok_r(domain_copy, ".", &tok_state);
+    buf_setcstr(&buf, "dc=");
 
     while (part != NULL) {
-	syslog(LOG_DEBUG, "Root DN now %s", buf);
-	strcat(buf, ",dc=");
-	syslog(LOG_DEBUG, "Root DN now %s", buf);
-	strcat(buf, part);
-	syslog(LOG_DEBUG, "Root DN now %s", buf);
-	part = strtok_r(NULL, ".", &ptr);
+	syslog(LOG_DEBUG, "Root DN now %s", buf_cstring(&buf));
+
+	buf_appendcstr(&buf, part);
+	syslog(LOG_DEBUG, "Root DN now %s", buf_cstring(&buf));
+
+	part = strtok_r(NULL, ".", &tok_state);
+	
+	if (part != NULL)
+	    buf_appendcstr(&buf, dc_sep);
     }
 
-    syslog(LOG_DEBUG, "Root DN now %s", buf);
+    free(domain_copy);
 
-    if (buf[0] == ',')
-	memmove(buf, buf+1, strlen(buf));
+    syslog(LOG_DEBUG, "Root DN now %s", buf_cstring(&buf));
 
-    *result = xstrdup(buf);
+    *result = buf_release(&buf);
 
-    free(buf);
-    free(part);
-/*    free(ptr);
-    free(root_dn_len);
-*/
     syslog(LOG_DEBUG, "Root DN now %s", *result);
 
     return PTSM_OK;
@@ -982,10 +967,8 @@ static int ptsmodule_get_dn(
 			ptsm->base = vals[0];
 			rc = PTSM_OK;
 		    } else if ((vals = ldap_get_values(ptsm->ld, entry, ptsm->domain_name_attribute)) != NULL) {
-			char *new_domain = xstrdup(vals[0]);
-			syslog(LOG_DEBUG, "Domain %s is an alias domain for parent domain %s", domain, new_domain);
-			rc = ptsmodule_standard_root_dn(new_domain, &ptsm->base);
-			free(new_domain);
+			syslog(LOG_DEBUG, "Domain %s is an alias domain for parent domain %s", domain, vals[0]);
+			rc = ptsmodule_standard_root_dn(vals[0], &ptsm->base);
 		    } else {
 			rc = ptsmodule_standard_root_dn(domain, &ptsm->base);
 		    }
