@@ -28,7 +28,7 @@ static int verify_flag(char *f)
     return 1;
 }
 
-int verify_flaglist(strarray_t *sl)
+EXPORTED int verify_flaglist(strarray_t *sl)
 {
     int i;
     char *joined;
