@@ -40,60 +40,42 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: timsieved.c,v 1.64 2009/12/16 02:51:45 brong Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <arpa/inet.h>
+#include <dirent.h>
+#include <limits.h>
+#include <netdb.h>
+#include <netinet/in.h>
 #include <sasl/sasl.h> /* yay! sasl */
-
+#include <signal.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <sys/stat.h>
-#include <errno.h>
-#include <stdlib.h>
-#include <limits.h>
-#include <sys/param.h>
-#include <syslog.h>
-#include <dirent.h>
-#include <ctype.h>
-#include <netdb.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <unistd.h>
-#include <signal.h>
 #include <string.h>
-
-#include "sieve_interface.h"
-
-#include "prot.h"
-#include "libconfig.h"
-#include "xmalloc.h"
-#include "exitcodes.h"
-#include "iptostring.h"
-#include "global.h"
-#include "codes.h"
-#include "actions.h"
-#include "parser.h"
-#include "lex.h"
-#include "mystring.h"
+#include <sys/param.h>
+#include <sys/socket.h>
+#include <sys/stat.h>
+#include <syslog.h>
+#include <unistd.h>
 
 #include "auth.h"
-#include "acl.h"
-#include "backend.h"
-#include "mboxlist.h"
-#include "proxy.h"
-#include "util.h"
-#include "xstrlcat.h"
-#include "xstrlcpy.h"
-
-#include "scripttest.h"
-
-#include "sync_log.h"
+#include "exitcodes.h"
+#include "libconfig.h"
+#include "xmalloc.h"
+#include "imap/backend.h"
+#include "imap/global.h"
+#include "imap/mboxlist.h"
+#include "imap/proxy.h"
+#include "imap/sync_log.h"
+#include "timsieved/actions.h"
+#include "timsieved/codes.h"
+#include "timsieved/parser.h"
+#include "timsieved/scripttest.h"
+#include "timsieved/lex.h"
 
 /* global state */
 const int config_need_data = 0;
@@ -108,10 +90,7 @@ static struct
 
 sasl_conn_t *sieved_saslconn; /* the sasl connection context */
 
-struct auth_state *sieved_authstate = 0;
-
-struct sockaddr_storage sieved_localaddr;
-struct sockaddr_storage sieved_remoteaddr;
+static struct auth_state *sieved_authstate = 0;
 
 int sieved_timeout;
 struct protstream *sieved_out;
@@ -119,8 +98,7 @@ struct protstream *sieved_in;
 
 int sieved_logfd = -1;
 
-int sieved_haveaddr = 0;
-char sieved_clienthost[NI_MAXHOST*2+1] = "[local]";
+const char *sieved_clienthost = "[local]";
 
 int sieved_userisadmin;
 int sieved_domainfromip = 0;
@@ -176,11 +154,14 @@ void shut_down(int code)
     exit(code);
 }
 
-void cmdloop()
+static void cmdloop(void)
 {
     int ret = FALSE;
     
-    chdir("/tmp/");
+    if (chdir("/tmp/")) {
+	syslog(LOG_ERR, "Failed to chdir to /tmp/");
+	ret = TRUE; /* exit immediately */
+    }
 
     capabilities(sieved_out, sieved_saslconn, 0, 0, 0);
 
@@ -206,14 +187,7 @@ void cmdloop()
     shut_down(0);
 }
 
-void printstring(const char *s __attribute__((unused)))
-{
-    /* needed to link against annotate.o */
-    fatal("printstring() executed, but its not used for timsieved!",
-	  EC_SOFTWARE);
-}
-
-void fatal(const char *s, int code)
+EXPORTED void fatal(const char *s, int code)
 {
     static int recurse_code = 0;
 
@@ -230,13 +204,13 @@ void fatal(const char *s, int code)
 }
 
 static struct sasl_callback mysasl_cb[] = {
-    { SASL_CB_GETOPT, &mysasl_config, NULL },
-    { SASL_CB_PROXY_POLICY, &mysasl_proxy_policy, (void*) &sieved_proxyctx },
-    { SASL_CB_CANON_USER, &mysasl_canon_user, (void*) &sieved_domainfromip },
+    { SASL_CB_GETOPT, (mysasl_cb_ft *) &mysasl_config, NULL },
+    { SASL_CB_PROXY_POLICY, (mysasl_cb_ft *) &mysasl_proxy_policy, (void*) &sieved_proxyctx },
+    { SASL_CB_CANON_USER, (mysasl_cb_ft *) &mysasl_canon_user, (void*) &sieved_domainfromip },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
-int service_init(int argc __attribute__((unused)),
+EXPORTED int service_init(int argc __attribute__((unused)),
 		 char **argv __attribute__((unused)),
 		 char **envp __attribute__((unused)))
 {
@@ -252,20 +226,17 @@ int service_init(int argc __attribute__((unused)),
 }
 
 /* Called by service API to shut down the service */
-void service_abort(int error)
+EXPORTED void service_abort(int error)
 {
     shut_down(error);
 }
 
-int service_main(int argc __attribute__((unused)),
+EXPORTED int service_main(int argc __attribute__((unused)),
 		 char **argv __attribute__((unused)),
 		 char **envp __attribute__((unused)))
 {
-    socklen_t salen;
-    char remoteip[60], localip[60];
+    const char *remoteip, *localip;
     sasl_security_properties_t *secprops = NULL;
-    char hbuf[NI_MAXHOST];
-    int niflags;
 
     sync_log_init();
 
@@ -284,32 +255,7 @@ int service_main(int argc __attribute__((unused)),
     if (geteuid() == 0) fatal("must run as the Cyrus user", -6);
 
     /* Find out name of client host */
-    salen = sizeof(sieved_remoteaddr);
-    if (getpeername(0, (struct sockaddr *)&sieved_remoteaddr, &salen) == 0 &&
-	(sieved_remoteaddr.ss_family == AF_INET ||
-	 sieved_remoteaddr.ss_family == AF_INET6)) {
-	if (getnameinfo((struct sockaddr *)&sieved_remoteaddr, salen,
-			hbuf, sizeof(hbuf), NULL, 0, NI_NAMEREQD) == 0) {
-	    strncpy(sieved_clienthost, hbuf, sizeof(hbuf));
-	} else {
-	    sieved_clienthost[0] = '\0';
-	}
-	niflags = NI_NUMERICHOST;
-#ifdef NI_WITHSCOPEID
-	if (((struct sockaddr *)&sieved_remoteaddr)->sa_family == AF_INET6)
-	    niflags |= NI_WITHSCOPEID;
-#endif
-	if (getnameinfo((struct sockaddr *)&sieved_remoteaddr, salen, hbuf,
-			sizeof(hbuf), NULL, 0, niflags) != 0)
-	    strlcpy(hbuf, "unknown", sizeof(hbuf));
-	strlcat(sieved_clienthost, "[", sizeof(sieved_clienthost));
-	strlcat(sieved_clienthost, hbuf, sizeof(sieved_clienthost));
-	strlcat(sieved_clienthost, "]", sizeof(sieved_clienthost));
-	salen = sizeof(sieved_localaddr);
-	if(getsockname(0, (struct sockaddr *)&sieved_localaddr, &salen) == 0) {
-	    sieved_haveaddr = 1;
-	}
-    }
+    sieved_clienthost = get_clienthost(0, &localip, &remoteip);
 
     /* other params should be filled in */
     if (sasl_server_new(SIEVE_SERVICE_NAME, config_servername, NULL,
@@ -317,13 +263,11 @@ int service_main(int argc __attribute__((unused)),
 			&sieved_saslconn) != SASL_OK)
 	fatal("SASL failed initializing: sasl_server_new()", -1); 
 
-    if(iptostring((struct sockaddr *)&sieved_remoteaddr,
-		  salen, remoteip, 60) == 0) {
+    if (remoteip) {
 	sasl_setprop(sieved_saslconn, SASL_IPREMOTEPORT, remoteip);
 	saslprops.ipremoteport = xstrdup(remoteip);
     }
-    if(iptostring((struct sockaddr *)&sieved_localaddr,
-		  salen, localip, 60) == 0) {
+    if (localip) {
 	sasl_setprop(sieved_saslconn, SASL_IPLOCALPORT, localip);
 	saslprops.iplocalport = xstrdup(localip);
     }
